@@ -1,4 +1,4 @@
-// Copyright (C) 2024 Daniel Mueller <deso@posteo.net>
+// Copyright (C) 2024-2025 Daniel Mueller <deso@posteo.net>
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
 
 //! A crate for interacting with [`debuginfod`][debuginfod] servers.
@@ -17,6 +17,9 @@ mod buildid;
 #[cfg(feature = "fs-cache")]
 mod caching_client;
 mod client;
+mod http_client;
+#[cfg(feature = "reqwest")]
+mod reqwest_client;
 mod util;
 
 pub use buildid::BuildId;
@@ -24,8 +27,15 @@ pub use buildid::BuildId;
 #[cfg_attr(docsrs, doc(cfg(feature = "fs-cache")))]
 pub use caching_client::CachingClient;
 pub use client::Client;
+pub use client::ClientBuilder;
 pub use client::Response;
+pub use http_client::HttpClient;
+pub use http_client::HttpClientError;
+pub use http_client::Readable;
 
+#[cfg(feature = "reqwest")]
+#[cfg_attr(docsrs, doc(cfg(feature = "reqwest")))]
+pub use reqwest;
 
 #[cfg(feature = "tracing")]
 #[macro_use]
