@@ -27,6 +27,7 @@ use bitflags::bitflags;
 use libbpf_sys::bpf_map_info;
 use libbpf_sys::bpf_obj_get_info_by_fd;
 
+use crate::error;
 use crate::util;
 use crate::util::parse_ret_i32;
 use crate::util::validate_bpf_ret;
@@ -42,7 +43,6 @@ pub type OpenMap<'obj> = OpenMapImpl<'obj>;
 /// A mutable parsed but not yet loaded BPF map.
 pub type OpenMapMut<'obj> = OpenMapImpl<'obj, Mut>;
 
-
 /// Represents a parsed but not yet loaded BPF map.
 ///
 /// This object exposes operations that need to happen before the map is created.
@@ -56,8 +56,6 @@ pub struct OpenMapImpl<'obj, T = ()> {
     _phantom: PhantomData<&'obj T>,
 }
 
-// TODO: Document members.
-#[allow(missing_docs)]
 impl<'obj> OpenMap<'obj> {
     /// Create a new [`OpenMap`] from a ptr to a `libbpf_sys::bpf_map`.
     pub fn new(object: &'obj libbpf_sys::bpf_map) -> Self {
@@ -70,7 +68,7 @@ impl<'obj> OpenMap<'obj> {
     }
 
     /// Retrieve the [`OpenMap`]'s name.
-    pub fn name(&self) -> &OsStr {
+    pub fn name(&self) -> &'obj OsStr {
         // SAFETY: We ensured `ptr` is valid during construction.
         let name_ptr = unsafe { libbpf_sys::bpf_map__name(self.ptr.as_ptr()) };
         // SAFETY: `bpf_map__name` can return NULL but only if it's passed
@@ -103,6 +101,16 @@ impl<'obj> OpenMap<'obj> {
             Some(data)
         }
     }
+
+    /// Retrieve the maximum number of entries of the map.
+    pub fn max_entries(&self) -> u32 {
+        unsafe { libbpf_sys::bpf_map__max_entries(self.ptr.as_ptr()) }
+    }
+
+    /// Return `true` if the map is set to be auto-created during load, `false` otherwise.
+    pub fn autocreate(&self) -> bool {
+        unsafe { libbpf_sys::bpf_map__autocreate(self.ptr.as_ptr()) }
+    }
 }
 
 impl<'obj> OpenMapMut<'obj> {
@@ -175,15 +183,19 @@ impl<'obj> OpenMapMut<'obj> {
         util::parse_ret(ret)
     }
 
-    // TODO: Document member.
-    #[allow(missing_docs)]
+    /// Set the NUMA node for this map.
+    ///
+    /// This can be used to ensure that the map is allocated on a particular
+    /// NUMA node, which can be useful for performance-critical applications.
     pub fn set_numa_node(&mut self, numa_node: u32) -> Result<()> {
         let ret = unsafe { libbpf_sys::bpf_map__set_numa_node(self.ptr.as_ptr(), numa_node) };
         util::parse_ret(ret)
     }
 
-    // TODO: Document member.
-    #[allow(missing_docs)]
+    /// Set the inner map FD.
+    ///
+    /// This is used for nested maps, where the value type of the outer map is a pointer to the
+    /// inner map.
     pub fn set_inner_map_fd(&mut self, inner_map_fd: BorrowedFd<'_>) -> Result<()> {
         let ret = unsafe {
             libbpf_sys::bpf_map__set_inner_map_fd(self.ptr.as_ptr(), inner_map_fd.as_raw_fd())
@@ -191,8 +203,14 @@ impl<'obj> OpenMapMut<'obj> {
         util::parse_ret(ret)
     }
 
-    // TODO: Document member.
-    #[allow(missing_docs)]
+    /// Set the `map_extra` field for this map.
+    ///
+    /// Allows users to pass additional data to the
+    /// kernel when loading the map. The kernel will store this value in the
+    /// `bpf_map_info` struct associated with the map.
+    ///
+    /// This can be used to pass data to the kernel that is not otherwise
+    /// representable via the existing `bpf_map_def` fields.
     pub fn set_map_extra(&mut self, map_extra: u64) -> Result<()> {
         let ret = unsafe { libbpf_sys::bpf_map__set_map_extra(self.ptr.as_ptr(), map_extra) };
         util::parse_ret(ret)
@@ -259,7 +277,7 @@ impl<T> AsRawLibbpf for OpenMapImpl<'_, T> {
 
 pub(crate) fn map_fd(map: NonNull<libbpf_sys::bpf_map>) -> Option<RawFd> {
     let fd = unsafe { libbpf_sys::bpf_map__fd(map.as_ptr()) };
-    let fd = util::parse_ret_i32(fd).ok().map(|fd| fd as RawFd);
+    let fd = util::parse_ret_i32(fd).ok();
     fd
 }
 
@@ -298,8 +316,14 @@ where
     key.as_ptr() as *const c_void
 }
 
-/// Internal function to return a value from a map into a buffer of the given size.
-fn lookup_raw<M>(map: &M, key: &[u8], flags: MapFlags, out_size: usize) -> Result<Option<Vec<u8>>>
+/// Internal function to perform a map lookup and write the value into raw pointer.
+/// Returns `Ok(true)` if the key was found, `Ok(false)` if not found, or an error.
+fn lookup_raw<M>(
+    map: &M,
+    key: &[u8],
+    value: &mut [mem::MaybeUninit<u8>],
+    flags: MapFlags,
+) -> Result<bool>
 where
     M: MapCore + ?Sized,
 {
@@ -309,34 +333,65 @@ where
             key.len(),
             map.key_size()
         )));
-    };
+    }
 
-    let mut out: Vec<u8> = Vec::with_capacity(out_size);
+    // Make sure the internal users of this function pass the expected buffer size
+    debug_assert_eq!(
+        value.len(),
+        if map.map_type().is_percpu() {
+            percpu_buffer_size(map).unwrap()
+        } else {
+            map.value_size() as usize
+        }
+    );
 
     let ret = unsafe {
         libbpf_sys::bpf_map_lookup_elem_flags(
             map.as_fd().as_raw_fd(),
             map_key(map, key),
-            out.as_mut_ptr() as *mut c_void,
+            // TODO: Use `MaybeUninit::slice_as_mut_ptr` once stable.
+            value.as_mut_ptr().cast(),
             flags.bits(),
         )
     };
 
     if ret == 0 {
-        unsafe {
-            out.set_len(out_size);
-        }
-        Ok(Some(out))
+        Ok(true)
     } else {
         let err = io::Error::last_os_error();
         if err.kind() == io::ErrorKind::NotFound {
-            Ok(None)
+            Ok(false)
         } else {
             Err(Error::from(err))
         }
     }
 }
 
+/// Internal function to return a value from a map into a buffer of the given size.
+fn lookup_raw_vec<M>(
+    map: &M,
+    key: &[u8],
+    flags: MapFlags,
+    out_size: usize,
+) -> Result<Option<Vec<u8>>>
+where
+    M: MapCore + ?Sized,
+{
+    // Allocate without initializing (avoiding memset)
+    let mut out = Vec::with_capacity(out_size);
+
+    match lookup_raw(map, key, out.spare_capacity_mut(), flags)? {
+        true => {
+            // SAFETY: `lookup_raw` successfully filled the buffer
+            unsafe {
+                out.set_len(out_size);
+            }
+            Ok(Some(out))
+        }
+        false => Ok(None),
+    }
+}
+
 /// Internal function to update a map. This does not check the length of the
 /// supplied value.
 fn update_raw<M>(map: &M, key: &[u8], value: &[u8], flags: MapFlags) -> Result<()>
@@ -363,6 +418,57 @@ where
     util::parse_ret(ret)
 }
 
+/// Internal function to batch lookup (and delete) elements from a map.
+fn lookup_batch_raw<M>(
+    map: &M,
+    count: u32,
+    elem_flags: MapFlags,
+    flags: MapFlags,
+    delete: bool,
+) -> BatchedMapIter<'_>
+where
+    M: MapCore + ?Sized,
+{
+    #[allow(clippy::needless_update)]
+    let opts = libbpf_sys::bpf_map_batch_opts {
+        sz: mem::size_of::<libbpf_sys::bpf_map_batch_opts>() as _,
+        elem_flags: elem_flags.bits(),
+        flags: flags.bits(),
+        // bpf_map_batch_opts might have padding fields on some platform
+        ..Default::default()
+    };
+
+    // for maps of type BPF_MAP_TYPE_{HASH, PERCPU_HASH, LRU_HASH, LRU_PERCPU_HASH}
+    // the key size must be at least 4 bytes
+    let key_size = if map.map_type().is_hash_map() {
+        map.key_size().max(4)
+    } else {
+        map.key_size()
+    };
+
+    BatchedMapIter::new(map.as_fd(), count, key_size, map.value_size(), opts, delete)
+}
+
+/// Intneral function that returns an error for per-cpu and bloom filter maps.
+fn check_not_bloom_or_percpu<M>(map: &M) -> Result<()>
+where
+    M: MapCore + ?Sized,
+{
+    if map.map_type().is_bloom_filter() {
+        return Err(Error::with_invalid_data(
+            "lookup_bloom_filter() must be used for bloom filter maps",
+        ));
+    }
+    if map.map_type().is_percpu() {
+        return Err(Error::with_invalid_data(format!(
+            "lookup_percpu() must be used for per-cpu maps (type of the map is {:?})",
+            map.map_type(),
+        )));
+    }
+
+    Ok(())
+}
+
 #[allow(clippy::wildcard_imports)]
 mod private {
     use super::*;
@@ -387,6 +493,9 @@ pub trait MapCore: Debug + AsFd + private::Sealed {
     /// Retrieve the size of the map's values.
     fn value_size(&self) -> u32;
 
+    /// Retrieve `max_entries` of the map.
+    fn max_entries(&self) -> u32;
+
     /// Fetch extra map information
     #[inline]
     fn info(&self) -> Result<MapInfo> {
@@ -408,25 +517,74 @@ pub trait MapCore: Debug + AsFd + private::Sealed {
     ///
     /// If the map is one of the per-cpu data structures, the function [`Self::lookup_percpu()`]
     /// must be used.
-    /// If the map is of type bloom_filter the function [`Self::lookup_bloom_filter()`] must be used
+    /// If the map is of type `bloom_filter` the function [`Self::lookup_bloom_filter()`] must be
+    /// used
     fn lookup(&self, key: &[u8], flags: MapFlags) -> Result<Option<Vec<u8>>> {
-        if self.map_type().is_bloom_filter() {
-            return Err(Error::with_invalid_data(
-                "lookup_bloom_filter() must be used for bloom filter maps",
-            ));
-        }
-        if self.map_type().is_percpu() {
+        check_not_bloom_or_percpu(self)?;
+        let out_size = self.value_size() as usize;
+        lookup_raw_vec(self, key, flags, out_size)
+    }
+
+    /// Looks up a map value into a pre-allocated buffer, avoiding allocation.
+    ///
+    /// This method provides a zero-allocation alternative to [`Self::lookup()`].
+    ///
+    /// `key` must have exactly [`Self::key_size()`] elements.
+    /// `value` must have exactly [`Self::value_size()`] elements.
+    ///
+    /// Returns `Ok(true)` if the key was found and the buffer was filled,
+    /// `Ok(false)` if the key was not found, or an error.
+    ///
+    /// If the map is one of the per-cpu data structures, this function cannot be used.
+    /// If the map is of type `bloom_filter`, this function cannot be used.
+    fn lookup_into(&self, key: &[u8], value: &mut [u8], flags: MapFlags) -> Result<bool> {
+        check_not_bloom_or_percpu(self)?;
+
+        if value.len() != self.value_size() as usize {
             return Err(Error::with_invalid_data(format!(
-                "lookup_percpu() must be used for per-cpu maps (type of the map is {:?})",
-                self.map_type(),
+                "value buffer size {} != {}",
+                value.len(),
+                self.value_size()
             )));
         }
 
-        let out_size = self.value_size() as usize;
-        lookup_raw(self, key, flags, out_size)
+        // SAFETY: `u8` and `MaybeUninit<u8>` have the same in-memory representation.
+        let value = unsafe {
+            slice::from_raw_parts_mut::<mem::MaybeUninit<u8>>(
+                value.as_mut_ptr().cast(),
+                value.len(),
+            )
+        };
+        lookup_raw(self, key, value, flags)
     }
 
-    /// Returns if the given value is likely present in bloom_filter as `bool`.
+    /// Returns many elements in batch mode from the map.
+    ///
+    /// `count` specifies the batch size.
+    fn lookup_batch(
+        &self,
+        count: u32,
+        elem_flags: MapFlags,
+        flags: MapFlags,
+    ) -> Result<BatchedMapIter<'_>> {
+        check_not_bloom_or_percpu(self)?;
+        Ok(lookup_batch_raw(self, count, elem_flags, flags, false))
+    }
+
+    /// Returns many elements in batch mode from the map.
+    ///
+    /// `count` specifies the batch size.
+    fn lookup_and_delete_batch(
+        &self,
+        count: u32,
+        elem_flags: MapFlags,
+        flags: MapFlags,
+    ) -> Result<BatchedMapIter<'_>> {
+        check_not_bloom_or_percpu(self)?;
+        Ok(lookup_batch_raw(self, count, elem_flags, flags, true))
+    }
+
+    /// Returns if the given value is likely present in `bloom_filter` as `bool`.
     ///
     /// `value` must have exactly [`Self::value_size()`] elements.
     fn lookup_bloom_filter(&self, value: &[u8]) -> Result<bool> {
@@ -465,7 +623,7 @@ pub trait MapCore: Debug + AsFd + private::Sealed {
         let aligned_val_size = percpu_aligned_value_size(self);
         let out_size = percpu_buffer_size(self)?;
 
-        let raw_res = lookup_raw(self, key, flags, out_size)?;
+        let raw_res = lookup_raw_vec(self, key, flags, out_size)?;
         if let Some(raw_vals) = raw_res {
             let mut out = Vec::new();
             for chunk in raw_vals.chunks_exact(aligned_val_size) {
@@ -754,7 +912,8 @@ impl<'obj> Map<'obj> {
         unsafe { libbpf_sys::bpf_map__is_pinned(self.ptr.as_ptr()) }
     }
 
-    /// Returns the pin_path if the map is pinned, otherwise, None is returned
+    /// Returns the `pin_path` if the map is pinned, otherwise, `None`
+    /// is returned.
     pub fn get_pin_path(&self) -> Option<&OsStr> {
         let path_ptr = unsafe { libbpf_sys::bpf_map__pin_path(self.ptr.as_ptr()) };
         if path_ptr.is_null() {
@@ -764,6 +923,11 @@ impl<'obj> Map<'obj> {
         let path_c_str = unsafe { CStr::from_ptr(path_ptr) };
         Some(OsStr::from_bytes(path_c_str.to_bytes()))
     }
+
+    /// Return `true` if the map was set to be auto-created during load, `false` otherwise.
+    pub fn autocreate(&self) -> bool {
+        unsafe { libbpf_sys::bpf_map__autocreate(self.ptr.as_ptr()) }
+    }
 }
 
 impl<'obj> MapMut<'obj> {
@@ -835,7 +999,7 @@ impl<T> AsFd for MapImpl<'_, T> {
         let fd = map_fd(self.ptr).unwrap();
         // SAFETY: `fd` is guaranteed to be valid for the lifetime of
         //         the created object.
-        let fd = unsafe { BorrowedFd::borrow_raw(fd as _) };
+        let fd = unsafe { BorrowedFd::borrow_raw(fd) };
         fd
     }
 }
@@ -868,6 +1032,11 @@ where
     fn value_size(&self) -> u32 {
         unsafe { libbpf_sys::bpf_map__value_size(self.ptr.as_ptr()) }
     }
+
+    #[inline]
+    fn max_entries(&self) -> u32 {
+        unsafe { libbpf_sys::bpf_map__max_entries(self.ptr.as_ptr()) }
+    }
 }
 
 impl AsRawLibbpf for Map<'_> {
@@ -882,7 +1051,7 @@ impl AsRawLibbpf for Map<'_> {
 
 /// A handle to a map. Handles can be duplicated and dropped.
 ///
-/// While possible to [created directly][MapHandle::create], in many cases it is
+/// While possible to [create directly][MapHandle::create], in many cases it is
 /// useful to create such a handle from an existing [`Map`]:
 /// ```no_run
 /// # use libbpf_rs::Map;
@@ -901,6 +1070,7 @@ pub struct MapHandle {
     ty: MapType,
     key_size: u32,
     value_size: u32,
+    max_entries: u32,
 }
 
 impl MapHandle {
@@ -948,6 +1118,7 @@ impl MapHandle {
             ty: map_type,
             key_size,
             value_size,
+            max_entries,
         })
     }
 
@@ -998,13 +1169,14 @@ impl MapHandle {
             ty: info.map_type(),
             key_size: info.info.key_size,
             value_size: info.info.value_size,
+            max_entries: info.info.max_entries,
         })
     }
 
     /// Freeze the map as read-only from user space.
     ///
     /// Entries from a frozen map can no longer be updated or deleted with the
-    /// bpf() system call. This operation is not reversible, and the map remains
+    /// `bpf()` system call. This operation is not reversible, and the map remains
     /// immutable from user space until its destruction. However, read and write
     /// permissions for BPF programs to the map remain unchanged.
     pub fn freeze(&self) -> Result<()> {
@@ -1050,6 +1222,11 @@ impl MapCore for MapHandle {
     fn value_size(&self) -> u32 {
         self.value_size
     }
+
+    #[inline]
+    fn max_entries(&self) -> u32 {
+        self.max_entries
+    }
 }
 
 impl AsFd for MapHandle {
@@ -1075,14 +1252,15 @@ where
             ty: other.map_type(),
             key_size: other.key_size(),
             value_size: other.value_size(),
+            max_entries: other.max_entries(),
         })
     }
 }
 
-impl TryFrom<&MapHandle> for MapHandle {
+impl TryFrom<&Self> for MapHandle {
     type Error = Error;
 
-    fn try_from(other: &MapHandle) -> Result<Self> {
+    fn try_from(other: &Self) -> Result<Self> {
         Ok(Self {
             fd: other
                 .as_fd()
@@ -1092,6 +1270,7 @@ impl TryFrom<&MapHandle> for MapHandle {
             ty: other.map_type(),
             key_size: other.key_size(),
             value_size: other.value_size(),
+            max_entries: other.max_entries(),
         })
     }
 }
@@ -1116,40 +1295,145 @@ bitflags! {
 #[non_exhaustive]
 #[repr(u32)]
 #[derive(Copy, Clone, PartialEq, Eq, Debug)]
-// TODO: Document members.
-#[allow(missing_docs)]
 pub enum MapType {
+    /// An unspecified map type.
     Unspec = libbpf_sys::BPF_MAP_TYPE_UNSPEC,
+    /// A general purpose Hash map storage type.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_hash.html) for more details.
     Hash = libbpf_sys::BPF_MAP_TYPE_HASH,
+    /// An Array map storage type.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_array.html) for more details.
     Array = libbpf_sys::BPF_MAP_TYPE_ARRAY,
+    /// A program array map which holds only the file descriptors to other eBPF programs. Used for
+    /// tail-calls.
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_PROG_ARRAY/) for more details.
     ProgArray = libbpf_sys::BPF_MAP_TYPE_PROG_ARRAY,
+    /// An array map which holds only the file descriptors to perf events.
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_PERF_EVENT_ARRAY/) for more details.
     PerfEventArray = libbpf_sys::BPF_MAP_TYPE_PERF_EVENT_ARRAY,
+    /// A Hash map with per CPU storage.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_hash.html#per-cpu-hashes) for more details.
     PercpuHash = libbpf_sys::BPF_MAP_TYPE_PERCPU_HASH,
+    /// An Array map with per CPU storage.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_array.html) for more details.
     PercpuArray = libbpf_sys::BPF_MAP_TYPE_PERCPU_ARRAY,
+    #[allow(missing_docs)]
     StackTrace = libbpf_sys::BPF_MAP_TYPE_STACK_TRACE,
+    #[allow(missing_docs)]
     CgroupArray = libbpf_sys::BPF_MAP_TYPE_CGROUP_ARRAY,
+    /// A Hash map with least recently used (LRU) eviction policy.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_hash.html#bpf-map-type-lru-hash-and-variants) for more details.
     LruHash = libbpf_sys::BPF_MAP_TYPE_LRU_HASH,
+    /// A Hash map with least recently used (LRU) eviction policy with per CPU storage.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_hash.html#per-cpu-hashes) for more details.
     LruPercpuHash = libbpf_sys::BPF_MAP_TYPE_LRU_PERCPU_HASH,
+    /// A Longest Prefix Match (LPM) algorithm based map.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_lpm_trie.html) for more details.
     LpmTrie = libbpf_sys::BPF_MAP_TYPE_LPM_TRIE,
+    /// A map in map storage.
+    /// One level of nesting is supported, where an outer map contains instances of a single type
+    /// of inner map.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_of_maps.html) for more details.
     ArrayOfMaps = libbpf_sys::BPF_MAP_TYPE_ARRAY_OF_MAPS,
+    /// A map in map storage.
+    /// One level of nesting is supported, where an outer map contains instances of a single type
+    /// of inner map.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_of_maps.html) for more details.
     HashOfMaps = libbpf_sys::BPF_MAP_TYPE_HASH_OF_MAPS,
+    /// An array map that uses the key as the index to lookup a reference to a net device.
+    /// Primarily used for XDP BPF Helper.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_devmap.html) for more details.
     Devmap = libbpf_sys::BPF_MAP_TYPE_DEVMAP,
+    /// An array map holds references to a socket descriptor.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_sockmap.html) for more details.
     Sockmap = libbpf_sys::BPF_MAP_TYPE_SOCKMAP,
+    /// A map that redirects raw XDP frames to another CPU.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_cpumap.html) for more details.
     Cpumap = libbpf_sys::BPF_MAP_TYPE_CPUMAP,
+    /// A map that redirects raw XDP frames to `AF_XDP` sockets (XSKs), a new type of address
+    /// family in the kernel that allows redirection of frames from a driver to user space
+    /// without having to traverse the full network stack.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_xskmap.html) for more details.
     Xskmap = libbpf_sys::BPF_MAP_TYPE_XSKMAP,
+    /// A Hash map that holds references to sockets via their socket descriptor.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_sockmap.html) for more details.
     Sockhash = libbpf_sys::BPF_MAP_TYPE_SOCKHASH,
+    /// Deprecated. Use `CGrpStorage` instead.
+    ///
+    /// A Local storage for cgroups.
+    /// Only available with `CONFIG_CGROUP_BPF` and to programs that attach to cgroups.
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_cgroup_storage.html) for more details.
     CgroupStorage = libbpf_sys::BPF_MAP_TYPE_CGROUP_STORAGE,
+    /// A Local storage for cgroups. Only available with `CONFIG_CGROUPS`.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_cgrp_storage.html) for more details.
+    /// See also [Difference between cgrp_storage and cgroup_storage](https://docs.kernel.org/bpf/map_cgrp_storage.html#difference-between-bpf-map-type-cgrp-storage-and-bpf-map-type-cgroup-storage)
+    CGrpStorage = libbpf_sys::BPF_MAP_TYPE_CGRP_STORAGE,
+    /// A map that holds references to sockets with `SO_REUSEPORT` option set.
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_REUSEPORT_SOCKARRAY/) for more details.
     ReuseportSockarray = libbpf_sys::BPF_MAP_TYPE_REUSEPORT_SOCKARRAY,
+    /// A per-CPU variant of [`BPF_MAP_TYPE_CGROUP_STORAGE`][`MapType::CgroupStorage`].
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_PERCPU_CGROUP_STORAGE) for more details.
     PercpuCgroupStorage = libbpf_sys::BPF_MAP_TYPE_PERCPU_CGROUP_STORAGE,
+    /// A FIFO storage.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_queue_stack.html) for more details.
     Queue = libbpf_sys::BPF_MAP_TYPE_QUEUE,
+    /// A LIFO storage.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_queue_stack.html) for more details.
     Stack = libbpf_sys::BPF_MAP_TYPE_STACK,
+    /// A socket-local storage.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_sk_storage.html) for more details.
     SkStorage = libbpf_sys::BPF_MAP_TYPE_SK_STORAGE,
+    /// A Hash map that uses the key as the index to lookup a reference to a net device.
+    /// Primarily used for XDP BPF Helper.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_devmap.html) for more details.
     DevmapHash = libbpf_sys::BPF_MAP_TYPE_DEVMAP_HASH,
+    /// A specialized map that act as implementations of "struct ops" structures defined in the
+    /// kernel.
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_STRUCT_OPS/) for more details.
     StructOps = libbpf_sys::BPF_MAP_TYPE_STRUCT_OPS,
+    /// A ring buffer map to efficiently send large amount of data.
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_RINGBUF/) for more details.
     RingBuf = libbpf_sys::BPF_MAP_TYPE_RINGBUF,
+    /// A storage map that holds data keyed on inodes.
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_INODE_STORAGE/) for more details.
     InodeStorage = libbpf_sys::BPF_MAP_TYPE_INODE_STORAGE,
+    /// A storage map that holds data keyed on tasks.
+    ///
+    /// Refer [documentation](https://docs.ebpf.io/linux/map-type/BPF_MAP_TYPE_TASK_STORAGE/) for more details.
     TaskStorage = libbpf_sys::BPF_MAP_TYPE_TASK_STORAGE,
+    /// Bloom filters are a space-efficient probabilistic data structure used to quickly test
+    /// whether an element exists in a set. In a bloom filter, false positives are possible
+    /// whereas false negatives are not.
+    ///
+    /// Refer the kernel [documentation](https://docs.kernel.org/bpf/map_bloom_filter.html) for more details.
     BloomFilter = libbpf_sys::BPF_MAP_TYPE_BLOOM_FILTER,
+    #[allow(missing_docs)]
     UserRingBuf = libbpf_sys::BPF_MAP_TYPE_USER_RINGBUF,
     /// We choose to specify our own "unknown" type here b/c it's really up to the kernel
     /// to decide if it wants to reject the map. If it accepts it, it just means whoever
@@ -1162,22 +1446,27 @@ impl MapType {
     pub fn is_percpu(&self) -> bool {
         matches!(
             self,
-            MapType::PercpuArray
-                | MapType::PercpuHash
-                | MapType::LruPercpuHash
-                | MapType::PercpuCgroupStorage
+            Self::PercpuArray | Self::PercpuHash | Self::LruPercpuHash | Self::PercpuCgroupStorage
+        )
+    }
+
+    /// Returns if the map is of one of the hashmap types.
+    pub fn is_hash_map(&self) -> bool {
+        matches!(
+            self,
+            Self::Hash | Self::PercpuHash | Self::LruHash | Self::LruPercpuHash
         )
     }
 
     /// Returns if the map is keyless map type as per documentation of libbpf
     /// Keyless map types are: Queues, Stacks and Bloom Filters
     fn is_keyless(&self) -> bool {
-        matches!(self, MapType::Queue | MapType::Stack | MapType::BloomFilter)
+        matches!(self, Self::Queue | Self::Stack | Self::BloomFilter)
     }
 
     /// Returns if the map is of bloom filter type
     pub fn is_bloom_filter(&self) -> bool {
-        MapType::BloomFilter.eq(self)
+        Self::BloomFilter.eq(self)
     }
 
     /// Detects if host kernel supports this BPF map type.
@@ -1238,7 +1527,7 @@ impl From<u32> for MapType {
 
 impl From<MapType> for u32 {
     fn from(value: MapType) -> Self {
-        value as u32
+        value as Self
     }
 }
 
@@ -1264,7 +1553,7 @@ impl Iterator for MapKeyIter<'_> {
     type Item = Vec<u8>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        let prev = self.prev.as_ref().map_or(ptr::null(), |p| p.as_ptr());
+        let prev = self.prev.as_ref().map_or(ptr::null(), Vec::as_ptr);
 
         let ret = unsafe {
             libbpf_sys::bpf_map_get_next_key(
@@ -1282,6 +1571,122 @@ impl Iterator for MapKeyIter<'_> {
     }
 }
 
+/// An iterator over batches of key value pairs of a BPF map.
+#[derive(Debug)]
+pub struct BatchedMapIter<'map> {
+    map_fd: BorrowedFd<'map>,
+    delete: bool,
+    count: usize,
+    key_size: usize,
+    value_size: usize,
+    keys: Vec<u8>,
+    values: Vec<u8>,
+    prev: Option<Vec<u8>>,
+    next: Vec<u8>,
+    batch_opts: libbpf_sys::bpf_map_batch_opts,
+    index: Option<usize>,
+}
+
+impl<'map> BatchedMapIter<'map> {
+    fn new(
+        map_fd: BorrowedFd<'map>,
+        count: u32,
+        key_size: u32,
+        value_size: u32,
+        batch_opts: libbpf_sys::bpf_map_batch_opts,
+        delete: bool,
+    ) -> Self {
+        Self {
+            map_fd,
+            delete,
+            count: count as usize,
+            key_size: key_size as usize,
+            value_size: value_size as usize,
+            keys: vec![0; (count * key_size) as usize],
+            values: vec![0; (count * value_size) as usize],
+            prev: None,
+            next: vec![0; key_size as usize],
+            batch_opts,
+            index: None,
+        }
+    }
+
+    fn lookup_next_batch(&mut self) {
+        let prev = self.prev.as_mut().map_or(ptr::null_mut(), Vec::as_mut_ptr);
+        let mut count = self.count as u32;
+
+        let ret = unsafe {
+            let lookup_fn = if self.delete {
+                libbpf_sys::bpf_map_lookup_and_delete_batch
+            } else {
+                libbpf_sys::bpf_map_lookup_batch
+            };
+            lookup_fn(
+                self.map_fd.as_raw_fd(),
+                prev.cast(),
+                self.next.as_mut_ptr().cast(),
+                self.keys.as_mut_ptr().cast(),
+                self.values.as_mut_ptr().cast(),
+                &mut count,
+                &self.batch_opts,
+            )
+        };
+
+        if let Err(e) = util::parse_ret(ret) {
+            match e.kind() {
+                // in this case we can trust the returned count value
+                error::ErrorKind::NotFound => {}
+                // retry with same input arguments
+                error::ErrorKind::Interrupted => {
+                    return self.lookup_next_batch();
+                }
+                _ => {
+                    self.index = None;
+                    return;
+                }
+            }
+        }
+
+        self.prev = Some(self.next.clone());
+        self.index = Some(0);
+
+        unsafe {
+            self.keys.set_len(self.key_size * count as usize);
+            self.values.set_len(self.value_size * count as usize);
+        }
+    }
+}
+
+impl Iterator for BatchedMapIter<'_> {
+    type Item = (Vec<u8>, Vec<u8>);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let load_next_batch = match self.index {
+            Some(index) => {
+                let batch_finished = index * self.key_size >= self.keys.len();
+                let last_batch = self.keys.len() < self.key_size * self.count;
+                batch_finished && !last_batch
+            }
+            None => true,
+        };
+
+        if load_next_batch {
+            self.lookup_next_batch();
+        }
+
+        let index = self.index?;
+        let key = self.keys.chunks_exact(self.key_size).nth(index)?.to_vec();
+        let val = self
+            .values
+            .chunks_exact(self.value_size)
+            .nth(index)?
+            .to_vec();
+
+        self.index = Some(index + 1);
+        Some((key, val))
+    }
+}
+
 /// A convenience wrapper for [`bpf_map_info`][libbpf_sys::bpf_map_info]. It
 /// provides the ability to retrieve the details of a certain map.
 #[derive(Debug)]
