@@ -6,6 +6,7 @@ use std::ffi::c_void;
 use std::ffi::CStr;
 use std::ffi::CString;
 use std::ffi::OsStr;
+use std::io::Read;
 use std::marker::PhantomData;
 use std::mem;
 use std::mem::size_of;
@@ -27,6 +28,7 @@ use std::time::Duration;
 use libbpf_sys::bpf_func_id;
 
 use crate::netfilter;
+use crate::streams::Stream;
 use crate::util;
 use crate::util::validate_bpf_ret;
 use crate::util::BpfObjectType;
@@ -65,6 +67,25 @@ pub struct UprobeOpts {
     pub _non_exhaustive: (),
 }
 
+/// Options to optionally be provided when attaching to a uprobe.
+#[derive(Clone, Debug, Default)]
+pub struct UprobeMultiOpts {
+    /// Optional, array of function symbols to attach to
+    pub syms: Vec<String>,
+    /// Optional, array of function addresses to attach to
+    pub offsets: Vec<usize>,
+    /// Optional, array of associated ref counter offsets
+    pub ref_ctr_offsets: Vec<usize>,
+    /// Optional, array of associated BPF cookies
+    pub cookies: Vec<u64>,
+    /// Create return uprobes
+    pub retprobe: bool,
+    /// Create session uprobes
+    pub session: bool,
+    #[doc(hidden)]
+    pub _non_exhaustive: (),
+}
+
 /// Options to optionally be provided when attaching to a USDT.
 #[derive(Clone, Debug, Default)]
 pub struct UsdtOpts {
@@ -81,7 +102,7 @@ impl From<UsdtOpts> for libbpf_sys::bpf_usdt_opts {
             _non_exhaustive,
         } = opts;
         #[allow(clippy::needless_update)]
-        libbpf_sys::bpf_usdt_opts {
+        Self {
             sz: size_of::<Self>() as _,
             usdt_cookie: cookie,
             // bpf_usdt_opts might have padding fields on some platform
@@ -107,7 +128,7 @@ impl From<KprobeOpts> for libbpf_sys::bpf_kprobe_opts {
         } = opts;
 
         #[allow(clippy::needless_update)]
-        libbpf_sys::bpf_kprobe_opts {
+        Self {
             sz: size_of::<Self>() as _,
             bpf_cookie: cookie,
             // bpf_kprobe_opts might have padding fields on some platform
@@ -149,7 +170,7 @@ impl From<PerfEventOpts> for libbpf_sys::bpf_perf_event_opts {
         } = opts;
 
         #[allow(clippy::needless_update)]
-        libbpf_sys::bpf_perf_event_opts {
+        Self {
             sz: size_of::<Self>() as _,
             bpf_cookie: cookie,
             force_ioctl_attach,
@@ -159,11 +180,112 @@ impl From<PerfEventOpts> for libbpf_sys::bpf_perf_event_opts {
     }
 }
 
+
+/// Options used when iterating over a map.
+#[derive(Clone, Debug)]
+pub struct MapIterOpts<'fd> {
+    /// The file descriptor of the map.
+    pub fd: BorrowedFd<'fd>,
+    #[doc(hidden)]
+    pub _non_exhaustive: (),
+}
+
+impl<'fd> MapIterOpts<'fd> {
+    /// Create a [`MapIterOpts`] object using the given file descriptor.
+    pub fn from_fd(fd: BorrowedFd<'fd>) -> Self {
+        Self {
+            fd,
+            _non_exhaustive: (),
+        }
+    }
+}
+
+
+/// Iteration order for cgroups.
+#[non_exhaustive]
+#[repr(u32)]
+#[derive(Clone, Debug, Default)]
+pub enum CgroupIterOrder {
+    /// Use the default iteration order.
+    #[default]
+    Default = libbpf_sys::BPF_CGROUP_ITER_ORDER_UNSPEC,
+    /// Process only a single object.
+    SelfOnly = libbpf_sys::BPF_CGROUP_ITER_SELF_ONLY,
+    /// Walk descendants in pre-order.
+    DescendantsPre = libbpf_sys::BPF_CGROUP_ITER_DESCENDANTS_PRE,
+    /// Walk descendants in post-order.
+    DescendantsPost = libbpf_sys::BPF_CGROUP_ITER_DESCENDANTS_POST,
+    /// Walk ancestors upward.
+    AncestorsUp = libbpf_sys::BPF_CGROUP_ITER_ANCESTORS_UP,
+}
+
+/// Options used when iterating over a cgroup.
+#[derive(Clone, Debug)]
+pub struct CgroupIterOpts<'fd> {
+    /// The file descriptor of the cgroup.
+    pub fd: BorrowedFd<'fd>,
+    /// The iteration order to use on the cgroup.
+    pub order: CgroupIterOrder,
+    #[doc(hidden)]
+    pub _non_exhaustive: (),
+}
+
+impl<'fd> CgroupIterOpts<'fd> {
+    /// Create a [`CgroupIterOpts`] object using the given file descriptor.
+    pub fn from_fd(fd: BorrowedFd<'fd>) -> Self {
+        Self {
+            fd,
+            order: CgroupIterOrder::default(),
+            _non_exhaustive: (),
+        }
+    }
+}
+
+
+/// Options to optionally be provided when attaching to an iterator.
+#[non_exhaustive]
+#[derive(Clone, Debug)]
+pub enum IterOpts<'fd> {
+    /// Iterate over a map.
+    Map(MapIterOpts<'fd>),
+    /// Iterate over a group.
+    Cgroup(CgroupIterOpts<'fd>),
+}
+
+impl From<IterOpts<'_>> for libbpf_sys::bpf_iter_link_info {
+    fn from(opts: IterOpts) -> Self {
+        let mut linkinfo = Self::default();
+        match opts {
+            IterOpts::Map(map_opts) => {
+                let MapIterOpts {
+                    fd,
+                    _non_exhaustive: (),
+                } = map_opts;
+
+                linkinfo.map.map_fd = fd.as_raw_fd() as _;
+            }
+            IterOpts::Cgroup(cgroup_opts) => {
+                let CgroupIterOpts {
+                    fd,
+                    order,
+                    _non_exhaustive: (),
+                } = cgroup_opts;
+
+                linkinfo.cgroup.cgroup_fd = fd.as_raw_fd() as _;
+                linkinfo.cgroup.order = order as libbpf_sys::bpf_cgroup_iter_order;
+            }
+        };
+        linkinfo
+    }
+}
+
+
 /// An immutable parsed but not yet loaded BPF program.
 pub type OpenProgram<'obj> = OpenProgramImpl<'obj>;
 /// A mutable parsed but not yet loaded BPF program.
 pub type OpenProgramMut<'obj> = OpenProgramImpl<'obj, Mut>;
 
+
 /// Represents a parsed but not yet loaded BPF program.
 ///
 /// This object exposes operations that need to happen before the program is loaded.
@@ -191,7 +313,7 @@ impl<'obj> OpenProgram<'obj> {
     }
 
     /// Retrieve the name of this `OpenProgram`.
-    pub fn name(&self) -> &OsStr {
+    pub fn name(&self) -> &'obj OsStr {
         let name_ptr = unsafe { libbpf_sys::bpf_program__name(self.ptr.as_ptr()) };
         let name_c_str = unsafe { CStr::from_ptr(name_ptr) };
         // SAFETY: `bpf_program__name` always returns a non-NULL pointer.
@@ -199,7 +321,7 @@ impl<'obj> OpenProgram<'obj> {
     }
 
     /// Retrieve the name of the section this `OpenProgram` belongs to.
-    pub fn section(&self) -> &OsStr {
+    pub fn section(&self) -> &'obj OsStr {
         // SAFETY: The program is always valid.
         let p = unsafe { libbpf_sys::bpf_program__section_name(self.ptr.as_ptr()) };
         // SAFETY: `bpf_program__section_name` will always return a non-NULL
@@ -227,11 +349,16 @@ impl<'obj> OpenProgram<'obj> {
     /// instructions will be CO-RE-relocated, BPF subprograms instructions will be appended, ldimm64
     /// instructions will have FDs embedded, etc. So instructions returned before load and after it
     /// might be quite different.
-    pub fn insns(&self) -> &[libbpf_sys::bpf_insn] {
+    pub fn insns(&self) -> &'obj [libbpf_sys::bpf_insn] {
         let count = self.insn_cnt();
         let ptr = unsafe { libbpf_sys::bpf_program__insns(self.ptr.as_ptr()) };
         unsafe { slice::from_raw_parts(ptr, count) }
     }
+
+    /// Return `true` if the bpf program is set to autoload, `false` otherwise.
+    pub fn autoload(&self) -> bool {
+        unsafe { libbpf_sys::bpf_program__autoload(self.ptr.as_ptr()) }
+    }
 }
 
 impl<'obj> OpenProgramMut<'obj> {
@@ -290,7 +417,7 @@ impl<'obj> OpenProgramMut<'obj> {
         unsafe { libbpf_sys::bpf_program__set_autoattach(self.ptr.as_ptr(), autoattach) };
     }
 
-    #[allow(missing_docs)]
+    #[expect(missing_docs)]
     pub fn set_attach_target(
         &mut self,
         attach_prog_fd: i32,
@@ -339,7 +466,7 @@ impl<T> AsRawLibbpf for OpenProgramImpl<'_, T> {
 #[repr(u32)]
 #[derive(Copy, Clone, Debug)]
 // TODO: Document variants.
-#[allow(missing_docs)]
+#[expect(missing_docs)]
 pub enum ProgramType {
     Unspec = 0,
     SocketFilter = libbpf_sys::BPF_PROG_TYPE_SOCKET_FILTER,
@@ -454,7 +581,7 @@ impl From<u32> for ProgramType {
 #[repr(u32)]
 #[derive(Clone, Debug)]
 // TODO: Document variants.
-#[allow(missing_docs)]
+#[expect(missing_docs)]
 pub enum ProgramAttachType {
     CgroupInetIngress = libbpf_sys::BPF_CGROUP_INET_INGRESS,
     CgroupInetEgress = libbpf_sys::BPF_CGROUP_INET_EGRESS,
@@ -636,7 +763,6 @@ pub type Program<'obj> = ProgramImpl<'obj>;
 /// A mutable loaded BPF program.
 pub type ProgramMut<'obj> = ProgramImpl<'obj, Mut>;
 
-
 /// Represents a loaded [`Program`].
 ///
 /// This struct is not safe to clone because the underlying libbpf resource cannot currently
@@ -663,7 +789,7 @@ impl<'obj> Program<'obj> {
     }
 
     /// Retrieve the name of this `Program`.
-    pub fn name(&self) -> &OsStr {
+    pub fn name(&self) -> &'obj OsStr {
         let name_ptr = unsafe { libbpf_sys::bpf_program__name(self.ptr.as_ptr()) };
         let name_c_str = unsafe { CStr::from_ptr(name_ptr) };
         // SAFETY: `bpf_program__name` always returns a non-NULL pointer.
@@ -671,7 +797,7 @@ impl<'obj> Program<'obj> {
     }
 
     /// Retrieve the name of the section this `Program` belongs to.
-    pub fn section(&self) -> &OsStr {
+    pub fn section(&self) -> &'obj OsStr {
         // SAFETY: The program is always valid.
         let p = unsafe { libbpf_sys::bpf_program__section_name(self.ptr.as_ptr()) };
         // SAFETY: `bpf_program__section_name` will always return a non-NULL
@@ -687,7 +813,7 @@ impl<'obj> Program<'obj> {
     }
 
     #[deprecated = "renamed to Program::fd_from_id"]
-    #[allow(missing_docs)]
+    #[expect(missing_docs)]
     #[inline]
     pub fn get_fd_by_id(id: u32) -> Result<OwnedFd> {
         Self::fd_from_id(id)
@@ -703,14 +829,6 @@ impl<'obj> Program<'obj> {
         Ok(unsafe { OwnedFd::from_raw_fd(fd) })
     }
 
-    // TODO: Remove once 0.25 is cut.
-    #[deprecated = "renamed to Program::id_from_fd"]
-    #[allow(missing_docs)]
-    #[inline]
-    pub fn get_id_by_fd(fd: BorrowedFd<'_>) -> Result<u32> {
-        Self::id_from_fd(fd)
-    }
-
     /// Returns program ID given a file descriptor.
     pub fn id_from_fd(fd: BorrowedFd<'_>) -> Result<u32> {
         let mut prog_info = libbpf_sys::bpf_prog_info::default();
@@ -787,7 +905,7 @@ impl<'obj> Program<'obj> {
     /// Gives read-only access to BPF program's underlying BPF instructions.
     ///
     /// Please see note in [`OpenProgram::insns`].
-    pub fn insns(&self) -> &[libbpf_sys::bpf_insn] {
+    pub fn insns(&self) -> &'obj [libbpf_sys::bpf_insn] {
         let count = self.insn_cnt();
         let ptr = unsafe { libbpf_sys::bpf_program__insns(self.ptr.as_ptr()) };
         unsafe { slice::from_raw_parts(ptr, count) }
@@ -942,6 +1060,119 @@ impl<'obj> ProgramMut<'obj> {
         Ok(link)
     }
 
+    /// Attach this program to multiple
+    /// [uprobes](https://www.kernel.org/doc/html/latest/trace/uprobetracer.html) at once.
+    pub fn attach_uprobe_multi(
+        &self,
+        pid: i32,
+        binary_path: impl AsRef<Path>,
+        func_pattern: impl AsRef<str>,
+        retprobe: bool,
+        session: bool,
+    ) -> Result<Link> {
+        let opts = UprobeMultiOpts {
+            syms: Vec::new(),
+            offsets: Vec::new(),
+            ref_ctr_offsets: Vec::new(),
+            cookies: Vec::new(),
+            retprobe,
+            session,
+            _non_exhaustive: (),
+        };
+
+        self.attach_uprobe_multi_with_opts(pid, binary_path, func_pattern, opts)
+    }
+
+    /// Attach this program to multiple
+    /// [uprobes](https://www.kernel.org/doc/html/latest/trace/uprobetracer.html)
+    /// at once, providing additional options.
+    pub fn attach_uprobe_multi_with_opts(
+        &self,
+        pid: i32,
+        binary_path: impl AsRef<Path>,
+        func_pattern: impl AsRef<str>,
+        opts: UprobeMultiOpts,
+    ) -> Result<Link> {
+        let path = util::path_to_cstring(binary_path)?;
+        let path_ptr = path.as_ptr();
+
+        let UprobeMultiOpts {
+            syms,
+            offsets,
+            ref_ctr_offsets,
+            cookies,
+            retprobe,
+            session,
+            _non_exhaustive,
+        } = opts;
+
+        let pattern = util::str_to_cstring(func_pattern.as_ref())?;
+        let pattern_ptr = pattern.as_ptr();
+
+        let syms_cstrings = syms
+            .iter()
+            .map(|s| util::str_to_cstring(s))
+            .collect::<Result<Vec<_>>>()?;
+        let syms_ptrs = syms_cstrings
+            .iter()
+            .map(|cs| cs.as_ptr())
+            .collect::<Vec<_>>();
+        let syms_ptr = if !syms_ptrs.is_empty() {
+            syms_ptrs.as_ptr()
+        } else {
+            ptr::null()
+        };
+        let offsets_ptr = if !offsets.is_empty() {
+            offsets.as_ptr()
+        } else {
+            ptr::null()
+        };
+        let ref_ctr_offsets_ptr = if !ref_ctr_offsets.is_empty() {
+            ref_ctr_offsets.as_ptr()
+        } else {
+            ptr::null()
+        };
+        let cookies_ptr = if !cookies.is_empty() {
+            cookies.as_ptr()
+        } else {
+            ptr::null()
+        };
+        let cnt = if !syms.is_empty() {
+            syms.len()
+        } else if !offsets.is_empty() {
+            offsets.len()
+        } else {
+            0
+        };
+
+        let c_opts = libbpf_sys::bpf_uprobe_multi_opts {
+            sz: size_of::<libbpf_sys::bpf_uprobe_multi_opts>() as _,
+            syms: syms_ptr.cast_mut(),
+            offsets: offsets_ptr.cast(),
+            ref_ctr_offsets: ref_ctr_offsets_ptr.cast(),
+            cookies: cookies_ptr.cast(),
+            cnt: cnt as libbpf_sys::size_t,
+            retprobe,
+            session,
+            ..Default::default()
+        };
+
+        let ptr = unsafe {
+            libbpf_sys::bpf_program__attach_uprobe_multi(
+                self.ptr.as_ptr(),
+                pid,
+                path_ptr,
+                pattern_ptr,
+                &c_opts as *const _,
+            )
+        };
+
+        let ptr = validate_bpf_ret(ptr).context("failed to attach uprobe multi")?;
+        // SAFETY: the pointer came from libbpf and has been checked for errors.
+        let link = unsafe { Link::new(ptr) };
+        Ok(link)
+    }
+
     /// Attach this program to a [kernel
     /// probe](https://www.kernel.org/doc/html/latest/trace/kprobetrace.html).
     pub fn attach_kprobe<T: AsRef<str>>(&self, retprobe: bool, func_name: T) -> Result<Link> {
@@ -1331,10 +1562,22 @@ impl<'obj> ProgramMut<'obj> {
     /// [BPF Iterator](https://www.kernel.org/doc/html/latest/bpf/bpf_iterators.html).
     /// The entry point of the program must be defined with `SEC("iter")` or `SEC("iter.s")`.
     pub fn attach_iter(&self, map_fd: BorrowedFd<'_>) -> Result<Link> {
-        let mut linkinfo = libbpf_sys::bpf_iter_link_info::default();
-        linkinfo.map.map_fd = map_fd.as_raw_fd() as _;
+        let map_opts = MapIterOpts {
+            fd: map_fd,
+            _non_exhaustive: (),
+        };
+        self.attach_iter_with_opts(IterOpts::Map(map_opts))
+    }
+
+    /// Attach this program to a
+    /// [BPF Iterator](https://www.kernel.org/doc/html/latest/bpf/bpf_iterators.html),
+    /// providing additional options.
+    ///
+    /// The entry point of the program must be defined with `SEC("iter")` or `SEC("iter.s")`.
+    pub fn attach_iter_with_opts(&self, opts: IterOpts<'_>) -> Result<Link> {
+        let mut linkinfo = libbpf_sys::bpf_iter_link_info::from(opts);
         let attach_opt = libbpf_sys::bpf_iter_attach_opts {
-            link_info: &mut linkinfo as *mut libbpf_sys::bpf_iter_link_info,
+            link_info: &raw mut linkinfo,
             link_info_len: size_of::<libbpf_sys::bpf_iter_link_info>() as _,
             sz: size_of::<libbpf_sys::bpf_iter_attach_opts>() as _,
             ..Default::default()
@@ -1415,6 +1658,16 @@ impl<'obj> ProgramMut<'obj> {
         };
         Ok(output)
     }
+
+    /// Get the stdout BPF stream of the program.
+    pub fn stdout(&self) -> impl Read + '_ {
+        Stream::new(self.as_fd(), Stream::BPF_STDOUT)
+    }
+
+    /// Get the stderr BPF stream of the program.
+    pub fn stderr(&self) -> impl Read + '_ {
+        Stream::new(self.as_fd(), Stream::BPF_STDERR)
+    }
 }
 
 impl<'obj> Deref for ProgramMut<'obj> {
