@@ -11,9 +11,11 @@
 //! ```
 
 use std::ffi::c_void;
+use std::ffi::CStr;
 use std::ffi::CString;
 use std::io;
 use std::mem::size_of_val;
+use std::mem::zeroed;
 use std::os::fd::AsFd;
 use std::os::fd::AsRawFd;
 use std::os::fd::BorrowedFd;
@@ -107,7 +109,7 @@ pub struct LineInfo {
 
 impl From<&libbpf_sys::bpf_line_info> for LineInfo {
     fn from(item: &libbpf_sys::bpf_line_info) -> Self {
-        LineInfo {
+        Self {
             insn_off: item.insn_off,
             file_name_off: item.file_name_off,
             line_off: item.line_off,
@@ -178,6 +180,11 @@ pub struct ProgramInfo {
     pub run_cnt: u64,
     /// Skipped BPF executions due to recursion or concurrent execution prevention.
     pub recursion_misses: u64,
+    /// Number of instructions that were verified by the verifier.
+    pub verified_insns: u32,
+    /// The struct is non-exhaustive and open to extension.
+    #[doc(hidden)]
+    pub _non_exhaustive: (),
 }
 
 /// An iterator for the information of loaded bpf programs.
@@ -391,7 +398,7 @@ impl ProgramInfo {
         };
         util::parse_ret(ret)?;
 
-        Ok(ProgramInfo {
+        Ok(Self {
             name: name.to_owned(),
             ty,
             tag: Tag(item.tag),
@@ -418,6 +425,8 @@ impl ProgramInfo {
             run_time_ns: item.run_time_ns,
             run_cnt: item.run_cnt,
             recursion_misses: item.recursion_misses,
+            verified_insns: item.verified_insns,
+            _non_exhaustive: (),
         })
     }
 }
@@ -561,7 +570,7 @@ impl BtfInfo {
         };
         util::parse_ret(ret)?;
 
-        Ok(BtfInfo {
+        Ok(Self {
             // SANITY: Our buffer contained space for a NUL byte and we set its
             //         contents to 0. Barring a `libbpf` bug a NUL byte will be
             //         present.
@@ -623,6 +632,11 @@ pub struct RawTracepointLinkInfo {
 pub struct TracingLinkInfo {
     /// Attach type of the tracing link.
     pub attach_type: ProgramAttachType,
+    /// Target object ID (`prog_id` for [`ProgramType::Ext`], otherwise
+    /// BTF object id).
+    pub target_obj_id: u32,
+    /// BTF type id inside the target object.
+    pub target_btf_id: u32,
 }
 
 /// Information about a cgroup link
@@ -721,6 +735,64 @@ pub struct UprobeMultiLinkInfo {
     pub pid: u32,
 }
 
+/// Information about a perf event link.
+#[derive(Debug, Clone)]
+pub struct PerfEventLinkInfo {
+    /// The specific type of perf event with decoded information.
+    pub event_type: PerfEventType,
+}
+
+/// Specific types of perf events with decoded information.
+#[derive(Debug, Clone)]
+pub enum PerfEventType {
+    /// A tracepoint event.
+    Tracepoint {
+        /// The tracepoint name.
+        name: Option<CString>,
+        /// Attach cookie value for this link.
+        cookie: u64,
+    },
+    /// A kprobe event (includes both kprobe and kretprobe).
+    Kprobe {
+        /// The function being probed.
+        func_name: Option<CString>,
+        /// Whether this is a return probe (kretprobe).
+        is_retprobe: bool,
+        /// Address of the probe.
+        addr: u64,
+        /// Offset from the function.
+        offset: u32,
+        /// Number of missed events.
+        missed: u64,
+        /// Cookie value for the kprobe.
+        cookie: u64,
+    },
+    /// A uprobe event (includes both uprobe and uretprobe).
+    Uprobe {
+        /// The absolute file path of the binary being probed.
+        file_name: Option<CString>,
+        /// Whether this is a return probe (uretprobe).
+        is_retprobe: bool,
+        /// Offset from the binary.
+        offset: u32,
+        /// Cookie value for the uprobe.
+        cookie: u64,
+        /// Offset of kernel reference counted USDT semaphore.
+        ref_ctr_offset: u64,
+    },
+    /// A perf event.
+    Event {
+        /// The specific event of the perf event type.
+        config: u64,
+        /// The perf event type.
+        event_type: u32,
+        /// Cookie value for the perf event program.
+        cookie: u64,
+    },
+    /// An unknown or unsupported perf event type.
+    Unknown(u32),
+}
+
 /// Information about BPF link types. Maps to the anonymous union in `struct bpf_link_info` in
 /// kernel uapi.
 #[derive(Debug, Clone)]
@@ -762,7 +834,10 @@ pub enum LinkTypeInfo {
     /// Link type for sockmap programs.
     SockMap(SockMapLinkInfo),
     /// Link type for perf-event programs.
-    PerfEvent,
+    ///
+    /// Contains information about the perf event configuration including type and config
+    /// which can be used to identify tracepoints, kprobes, uprobes, etc.
+    PerfEvent(PerfEventLinkInfo),
     /// Unknown link type.
     Unknown,
 }
@@ -779,6 +854,22 @@ pub struct LinkInfo {
 }
 
 impl LinkInfo {
+    /// Create a `LinkInfo` object from a fd.
+    pub fn from_fd(fd: BorrowedFd<'_>) -> Result<Self> {
+        // See comment in gen_info_impl!() for why we use std::mem::zeroed()
+        let mut link_info: libbpf_sys::bpf_link_info = unsafe { zeroed() };
+        let item_ptr: *mut libbpf_sys::bpf_link_info = &mut link_info;
+        let mut len = size_of_val(&link_info) as u32;
+
+        let ret = unsafe {
+            libbpf_sys::bpf_obj_get_info_by_fd(fd.as_raw_fd(), item_ptr as *mut c_void, &mut len)
+        };
+        util::parse_ret(ret)?;
+
+        Self::from_uapi(fd, link_info)
+            .ok_or_else(|| crate::Error::with_invalid_data("failed to parse link info"))
+    }
+
     fn from_uapi(fd: BorrowedFd<'_>, mut s: libbpf_sys::bpf_link_info) -> Option<Self> {
         let type_info = match s.type_ {
             libbpf_sys::BPF_LINK_TYPE_RAW_TRACEPOINT => {
@@ -810,6 +901,8 @@ impl LinkInfo {
                 attach_type: ProgramAttachType::from(unsafe {
                     s.__bindgen_anon_1.tracing.attach_type
                 }),
+                target_obj_id: unsafe { s.__bindgen_anon_1.tracing.target_obj_id },
+                target_btf_id: unsafe { s.__bindgen_anon_1.tracing.target_btf_id },
             }),
             libbpf_sys::BPF_LINK_TYPE_CGROUP => LinkTypeInfo::Cgroup(CgroupLinkInfo {
                 cgroup_id: unsafe { s.__bindgen_anon_1.cgroup.cgroup_id },
@@ -867,7 +960,155 @@ impl LinkInfo {
                     s.__bindgen_anon_1.sockmap.attach_type
                 }),
             }),
-            libbpf_sys::BPF_LINK_TYPE_PERF_EVENT => LinkTypeInfo::PerfEvent,
+            libbpf_sys::BPF_LINK_TYPE_PERF_EVENT => {
+                // Get the BPF perf event type (BPF_PERF_EVENT_*) from the link info.
+                let bpf_perf_event_type = unsafe { s.__bindgen_anon_1.perf_event.type_ };
+
+                // Handle two-phase call for perf event string data if needed (this mimics the
+                // behavior of bpftool):
+                // For tracepoints, kprobes, and uprobes, we need to pass in a buffer to get the
+                // name. So we initialize the struct with a buffer pointer, and call
+                // `bpf_obj_get_info_by_fd` again to populate the name.
+                let mut buf = [0u8; libc::PATH_MAX as usize];
+                let call_get_info_again = match bpf_perf_event_type {
+                    libbpf_sys::BPF_PERF_EVENT_TRACEPOINT => {
+                        s.__bindgen_anon_1
+                            .perf_event
+                            .__bindgen_anon_1
+                            .tracepoint
+                            .tp_name = buf.as_mut_ptr() as u64;
+                        s.__bindgen_anon_1
+                            .perf_event
+                            .__bindgen_anon_1
+                            .tracepoint
+                            .name_len = buf.len() as u32;
+                        true
+                    }
+                    libbpf_sys::BPF_PERF_EVENT_KPROBE | libbpf_sys::BPF_PERF_EVENT_KRETPROBE => {
+                        s.__bindgen_anon_1
+                            .perf_event
+                            .__bindgen_anon_1
+                            .kprobe
+                            .func_name = buf.as_mut_ptr() as u64;
+                        s.__bindgen_anon_1
+                            .perf_event
+                            .__bindgen_anon_1
+                            .kprobe
+                            .name_len = buf.len() as u32;
+                        true
+                    }
+                    libbpf_sys::BPF_PERF_EVENT_UPROBE | libbpf_sys::BPF_PERF_EVENT_URETPROBE => {
+                        // SAFETY: This field is valid to access in `bpf_link_info`.
+                        let uprobe =
+                            unsafe { &mut s.__bindgen_anon_1.perf_event.__bindgen_anon_1.uprobe };
+                        uprobe.file_name = buf.as_mut_ptr() as u64;
+                        uprobe.name_len = buf.len() as u32;
+                        true
+                    }
+                    _ => false,
+                };
+
+                if call_get_info_again {
+                    let item_ptr: *mut libbpf_sys::bpf_link_info = &mut s;
+                    let mut len = size_of_val(&s) as u32;
+                    let ret = unsafe {
+                        libbpf_sys::bpf_obj_get_info_by_fd(
+                            fd.as_raw_fd(),
+                            item_ptr as *mut c_void,
+                            &mut len,
+                        )
+                    };
+                    if ret != 0 {
+                        return None;
+                    }
+                }
+
+                let event_type = match bpf_perf_event_type {
+                    libbpf_sys::BPF_PERF_EVENT_TRACEPOINT => {
+                        let tp_name = unsafe {
+                            s.__bindgen_anon_1
+                                .perf_event
+                                .__bindgen_anon_1
+                                .tracepoint
+                                .tp_name
+                        };
+                        let cookie = unsafe {
+                            s.__bindgen_anon_1
+                                .perf_event
+                                .__bindgen_anon_1
+                                .tracepoint
+                                .cookie
+                        };
+                        let name = (tp_name != 0).then(|| unsafe {
+                            CStr::from_ptr(tp_name as *const c_char).to_owned()
+                        });
+
+                        PerfEventType::Tracepoint { name, cookie }
+                    }
+                    libbpf_sys::BPF_PERF_EVENT_KPROBE | libbpf_sys::BPF_PERF_EVENT_KRETPROBE => {
+                        let func_name = unsafe {
+                            s.__bindgen_anon_1
+                                .perf_event
+                                .__bindgen_anon_1
+                                .kprobe
+                                .func_name
+                        };
+                        let addr =
+                            unsafe { s.__bindgen_anon_1.perf_event.__bindgen_anon_1.kprobe.addr };
+                        let offset =
+                            unsafe { s.__bindgen_anon_1.perf_event.__bindgen_anon_1.kprobe.offset };
+                        let missed =
+                            unsafe { s.__bindgen_anon_1.perf_event.__bindgen_anon_1.kprobe.missed };
+                        let cookie =
+                            unsafe { s.__bindgen_anon_1.perf_event.__bindgen_anon_1.kprobe.cookie };
+                        let func_name = (func_name != 0).then(|| unsafe {
+                            CStr::from_ptr(func_name as *const c_char).to_owned()
+                        });
+
+                        let is_retprobe =
+                            bpf_perf_event_type == libbpf_sys::BPF_PERF_EVENT_KRETPROBE;
+                        PerfEventType::Kprobe {
+                            func_name,
+                            is_retprobe,
+                            addr,
+                            offset,
+                            missed,
+                            cookie,
+                        }
+                    }
+                    libbpf_sys::BPF_PERF_EVENT_UPROBE | libbpf_sys::BPF_PERF_EVENT_URETPROBE => {
+                        // SAFETY: This field is valid to access in `bpf_link_info`.
+                        let uprobe =
+                            unsafe { s.__bindgen_anon_1.perf_event.__bindgen_anon_1.uprobe };
+                        // SAFETY: `file_name_ptr` is a valid nul terminated string pointer.
+                        let file_name = (uprobe.file_name != 0).then(|| unsafe {
+                            CStr::from_ptr(uprobe.file_name as *const c_char).to_owned()
+                        });
+
+                        PerfEventType::Uprobe {
+                            file_name,
+                            is_retprobe: bpf_perf_event_type
+                                == libbpf_sys::BPF_PERF_EVENT_URETPROBE,
+                            offset: uprobe.offset,
+                            cookie: uprobe.cookie,
+                            ref_ctr_offset: uprobe.ref_ctr_offset,
+                        }
+                    }
+                    libbpf_sys::BPF_PERF_EVENT_EVENT => {
+                        // SAFETY: This field is valid to access in `bpf_link_info`.
+                        let event = unsafe { s.__bindgen_anon_1.perf_event.__bindgen_anon_1.event };
+
+                        PerfEventType::Event {
+                            config: event.config,
+                            event_type: event.type_,
+                            cookie: event.cookie,
+                        }
+                    }
+                    ty => PerfEventType::Unknown(ty),
+                };
+
+                LinkTypeInfo::PerfEvent(PerfEventLinkInfo { event_type })
+            }
             _ => LinkTypeInfo::Unknown,
         };
 
